import os
from collections import OrderedDict
from typing import Optional, Union

from transformers import PretrainedConfig

"""Returns exaone to llama mapping function"""


def get_exaone_mapping_func():
    return {
        "transformer": "model",
        ".h.": ".layers.",
        "ln_1": "input_layernorm",
        "ln_2": "post_attention_layernorm",
        "ln_f": "norm",
        "wte": "embed_tokens",
        "c_fc_0": "gate_proj",
        "c_fc_1": "up_proj",
        "c_proj": "down_proj",
        "rotary": "rotary_emb",
        ".attn.attention.": ".self_attn.",
        "out_proj": "o_proj",
    }


"""Retrieves exaone model information and converts ckpt using exaone_mapping_func"""


def get_exaone_infos(
    pretrained_model_name_or_path: Optional[Union[str, os.PathLike]],
    *model_args,
    config: Optional[Union[PretrainedConfig, str, os.PathLike]] = None,
    cache_dir: Optional[Union[str, os.PathLike]] = None,
    ignore_mismatched_sizes: bool = False,
    force_download: bool = False,
    local_files_only: bool = False,
    token: Optional[Union[str, bool]] = None,
    revision: str = "main",
    use_safetensors: bool = None,
    **kwargs,
):
    if pretrained_model_name_or_path not in [
        "LGAI-EXAONE/EXAONE-3.0-7.8B-Instruct",
        "LGAI-EXAONE/EXAONE-3.5-2.4B-Instruct",
        "LGAI-EXAONE/EXAONE-3.5-7.8B-Instruct",
        "LGAI-EXAONE/EXAONE-3.5-32B-Instruct",
    ]:
        return {
            "torch_dtype": None,
            "conversion_func": dict(),
            "original_config": None,
            "exaone_state_dict": OrderedDict(),
        }
    else:
        from transformers import AutoModelForCausalLM as HFAutoModelForCausalLM

        conversion_func = dict()
        hf_model = HFAutoModelForCausalLM.from_pretrained(
            pretrained_model_name_or_path=pretrained_model_name_or_path,
            *model_args,
            config=config,
            cache_dir=cache_dir,
            ignore_mismatched_sizes=ignore_mismatched_sizes,
            force_download=force_download,
            local_files_only=local_files_only,
            token=token,
            revision=revision,
            use_safetensors=use_safetensors,
            **kwargs,
        )

        state_dict = hf_model.state_dict()
        mapping_func = get_exaone_mapping_func()

        keys = list(state_dict.keys())
        for old_key in keys:
            new_key = old_key
            for key in mapping_func:
                new_key = new_key.replace(key, mapping_func[key])
            if new_key != old_key:
                state_dict[new_key] = state_dict.pop(old_key)
            conversion_func[old_key] = new_key
        return {
            "torch_dtype": hf_model.dtype,
            "conversion_func": conversion_func,
            "original_config": hf_model.config,
            "exaone_state_dict": state_dict,
        }
